"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addValidator_rawSignatures = exports.addValidator_unsignedHashes = exports.addValidator = void 0;
const dist_1 = require("@flarenetwork/flarejs/dist");
const platformvm_1 = require("@flarenetwork/flarejs/dist/apis/platformvm");
const utils_1 = require("@flarenetwork/flarejs/dist/utils");
const utils_2 = require("./utils");
/**
 * Stake by registring your node for validation
 * @param ctx - context with constants initialized from user keys
 * @param nodeID - id of the node you are running (can get it via rpc call)
 * @param stakeAmount - the amount of funds to stake during the node's validation
 * @param startTime - start time of the node's validation
 * @param endTime - end time of the node's validation
 */
function addValidator(ctx, nodeID, stakeAmount, startTime, endTime) {
    return __awaiter(this, void 0, void 0, function* () {
        const threshold = 1;
        const locktime = new dist_1.BN(0);
        const memo = dist_1.Buffer.from('PlatformVM utility method buildAddValidatorTx to add a validator to the primary subnet');
        const asOf = (0, utils_1.UnixNow)();
        const delegationFee = 10;
        // const stakeAmount: any = (await pchain.getMinStake()).minValidatorStake
        const platformVMUTXOResponse = yield ctx.pchain.getUTXOs(ctx.pAddressBech32);
        const utxoSet = platformVMUTXOResponse.utxos;
        const unsignedTx = yield ctx.pchain.buildAddValidatorTx(utxoSet, [ctx.pAddressBech32], [ctx.pAddressBech32], [ctx.pAddressBech32], nodeID, startTime, endTime, stakeAmount, [ctx.pAddressBech32], delegationFee, locktime, threshold, memo, asOf);
        const tx = unsignedTx.sign(ctx.pKeychain);
        const txid = yield ctx.pchain.issueTx(tx);
        return { txid: txid };
    });
}
exports.addValidator = addValidator;
/**
 * Get hashes that need to get signed for the addValidator transaction
 * @param ctx - context with constants initialized from user keys
 * @param nodeID - id of the node you are running (can get it via rpc call)
 * @param stakeAmount - the amount of funds to stake during the node's validation
 * @param startTime - start time of the node's validation
 * @param endTime - end time of the node's validation
 */
function addValidator_unsignedHashes(ctx, id, nodeID, stakeAmount, startTime, endTime) {
    return __awaiter(this, void 0, void 0, function* () {
        const threshold = 1;
        const locktime = new dist_1.BN(0);
        const memo = dist_1.Buffer.from('PlatformVM utility method buildAddValidatorTx to add a validator to the primary subnet');
        const asOf = (0, utils_1.UnixNow)();
        const delegationFee = 10;
        const platformVMUTXOResponse = yield ctx.pchain.getUTXOs(ctx.pAddressBech32);
        const unsignedTx = yield ctx.pchain.buildAddValidatorTx(platformVMUTXOResponse.utxos, [ctx.pAddressBech32], [ctx.pAddressBech32], [ctx.pAddressBech32], nodeID, startTime, endTime, stakeAmount, [ctx.pAddressBech32], delegationFee, locktime, threshold, memo, asOf);
        const unsignedTxJson = {
            serialization: (0, utils_2.serializeUnsignedTx)(unsignedTx),
            signatureRequests: unsignedTx.prepareUnsignedHashes(ctx.cKeychain),
            unsignedTransactionBuffer: unsignedTx.toBuffer().toString('hex')
        };
        (0, utils_2.saveUnsignedTx)(unsignedTxJson, id);
        return unsignedTxJson.signatureRequests;
    });
}
exports.addValidator_unsignedHashes = addValidator_unsignedHashes;
/**
 * Execute add validator transaction by providing signed hashes
 * @param ctx - context with constants initialized from user keys
 * @param signatures - signatures of the relevant hashes
 * @param transaction - serialized addValidator transaction
 */
function addValidator_rawSignatures(ctx, signatures, id) {
    return __awaiter(this, void 0, void 0, function* () {
        const unsignedTxJson = (0, utils_2.readUnsignedTx)(id);
        if (signatures.length !== unsignedTxJson.signatureRequests.length) {
            signatures = Array(unsignedTxJson.signatureRequests.length).fill(signatures[0]);
        }
        const ecdsaSignatures = signatures.map((signature) => (0, utils_2.expandSignature)(signature));
        const unsignedTx = (0, utils_2.deserializeUnsignedTx)(platformvm_1.UnsignedTx, unsignedTxJson.serialization);
        const tx = unsignedTx.signWithRawSignatures(ecdsaSignatures, ctx.cKeychain);
        const txid = yield ctx.pchain.issueTx(tx);
        return { txid: txid };
    });
}
exports.addValidator_rawSignatures = addValidator_rawSignatures;
