"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.importTxCP_rawSignatures = exports.importTxCP_unsignedHashes = exports.exportTxPC = exports.importTxCP = void 0;
const dist_1 = require("@flarenetwork/flarejs/dist");
const platformvm_1 = require("@flarenetwork/flarejs/dist/apis/platformvm");
const utils_1 = require("@flarenetwork/flarejs/dist//utils");
const utils_2 = require("./utils");
/**
 * Import funds exported from C-chain to P-chain to P-chain
 * @param ctx - context with constants initialized from user keys
 */
function importTxCP(ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        const threshold = 1;
        const locktime = new dist_1.BN(0);
        const memo = dist_1.Buffer.from('PlatformVM utility method buildImportTx to import AVAX to the P-Chain from the C-Chain');
        const asOf = (0, utils_1.UnixNow)();
        const platformVMUTXOResponse = yield ctx.pchain.getUTXOs([ctx.pAddressBech32], ctx.cChainBlockchainID);
        const utxoSet = platformVMUTXOResponse.utxos;
        const unsignedTx = yield ctx.pchain.buildImportTx(utxoSet, [ctx.pAddressBech32], ctx.cChainBlockchainID, [ctx.pAddressBech32], [ctx.pAddressBech32], [ctx.pAddressBech32], memo, asOf, locktime, threshold);
        const tx = unsignedTx.sign(ctx.pKeychain);
        const txid = yield ctx.pchain.issueTx(tx);
        return { txid: txid };
    });
}
exports.importTxCP = importTxCP;
/**
 * Export funds from P-chain to C-chain.
 * @param ctx - context with constants initialized from user keys
 * @param amount - amount to export (if left undefined, it exports all funds on P-chain)
 */
function exportTxPC(ctx, amount) {
    return __awaiter(this, void 0, void 0, function* () {
        const threshold = 1;
        const locktime = new dist_1.BN(0);
        const memo = dist_1.Buffer.from("PlatformVM utility method buildExportTx to export AVAX from the P-Chain to the C-Chain");
        const asOf = (0, utils_1.UnixNow)();
        const platformVMUTXOResponse = yield ctx.pchain.getUTXOs([ctx.pAddressBech32]);
        const utxoSet = platformVMUTXOResponse.utxos;
        const fee = ctx.pchain.getDefaultTxFee();
        if (amount === undefined) {
            const getBalanceResponse = yield ctx.pchain.getBalance(ctx.pAddressBech32);
            const unlocked = new dist_1.BN(getBalanceResponse.unlocked);
            amount = unlocked.sub(fee);
        }
        const unsignedTx = yield ctx.pchain.buildExportTx(utxoSet, amount, ctx.cChainBlockchainID, [ctx.cAddressBech32], [ctx.pAddressBech32], [ctx.pAddressBech32], memo, asOf, locktime, threshold);
        const tx = unsignedTx.sign(ctx.pKeychain);
        const txid = yield ctx.pchain.issueTx(tx);
        return { txid: txid };
    });
}
exports.exportTxPC = exportTxPC;
/**
 * Get hashes that need to get signed in order for funds exported from
 * C-chain to P-chain to be imported to P-chain
 * @param ctx - context with constants initialized from user keys
 */
function importTxCP_unsignedHashes(ctx, id) {
    return __awaiter(this, void 0, void 0, function* () {
        const threshold = 1;
        const locktime = new dist_1.BN(0);
        const memo = dist_1.Buffer.from('PlatformVM utility method buildImportTx to import AVAX to the P-Chain from the C-Chain');
        const asOf = (0, utils_1.UnixNow)();
        const platformVMUTXOResponse = yield ctx.pchain.getUTXOs([ctx.pAddressBech32], ctx.cChainBlockchainID);
        const unsignedTx = yield ctx.pchain.buildImportTx(platformVMUTXOResponse.utxos, [ctx.pAddressBech32], ctx.cChainBlockchainID, [ctx.pAddressBech32], [ctx.pAddressBech32], [ctx.pAddressBech32], memo, asOf, locktime, threshold);
        const unsignedTxJson = {
            serialization: (0, utils_2.serializeUnsignedTx)(unsignedTx),
            signatureRequests: unsignedTx.prepareUnsignedHashes(ctx.cKeychain),
            unsignedTransactionBuffer: unsignedTx.toBuffer().toString('hex')
        };
        (0, utils_2.saveUnsignedTx)(unsignedTxJson, id);
        return { signatureRequests: unsignedTxJson.signatureRequests };
    });
}
exports.importTxCP_unsignedHashes = importTxCP_unsignedHashes;
/**
 * Import funds from C-chain to P-chain by providing signed hashes
 * @param ctx - context with constants initialized from user keys
 * @param signatures - signatures of the relevant hashes
 * @param transaction - serialized import C - P transaction
 */
function importTxCP_rawSignatures(ctx, signatures, id) {
    return __awaiter(this, void 0, void 0, function* () {
        const unsignedTxJson = (0, utils_2.readUnsignedTx)(id);
        if (signatures.length === 0) {
            signatures = [(0, utils_2.readSignedTx)(id).signature];
        }
        if (signatures.length !== unsignedTxJson.signatureRequests.length) {
            signatures = Array(unsignedTxJson.signatureRequests.length).fill(signatures[0]);
        }
        const ecdsaSignatures = signatures.map((signature) => (0, utils_2.expandSignature)(signature));
        const unsignedTx = (0, utils_2.deserializeUnsignedTx)(platformvm_1.UnsignedTx, unsignedTxJson.serialization);
        const tx = unsignedTx.signWithRawSignatures(ecdsaSignatures, ctx.cKeychain);
        const chainTxId = yield ctx.pchain.issueTx(tx);
        return { chainTxId: chainTxId };
    });
}
exports.importTxCP_rawSignatures = importTxCP_rawSignatures;
